from OFS.SimpleItem import SimpleItem
from plone.app.z3cform.widgets.datetime import DateFieldWidget
from plone.formwidget.recurrence.tests.base import IntegrationTestCase
from plone.formwidget.recurrence.z3cform.widget import RecurrenceFieldWidget
from z3c.form import field
from z3c.form import form
from z3c.form.testing import TestRequest
from zope.schema.fieldproperty import FieldProperty

import json
import zope.interface
import zope.schema


class ITestForm(zope.interface.Interface):
    recurrence = zope.schema.Text(title="Recurrence", required=True)
    day = zope.schema.Date(title="Day", required=True)


@zope.interface.implementer(ITestForm)
class TestForm(SimpleItem):
    recurrence = FieldProperty(ITestForm["recurrence"])
    day = FieldProperty(ITestForm["day"])

    def __init__(self, recurrence, day):
        super().__init__(id)
        self.recurrence = recurrence
        self.day = day


class TestAddForm(form.AddForm):
    fields = field.Fields(ITestForm)
    fields["recurrence"].widgetFactory = RecurrenceFieldWidget
    fields["day"].widgetFactory = DateFieldWidget


class Z3CWidgetTestCase(IntegrationTestCase):
    def setUp(self):
        self.portal = self.layer["portal"]
        request = TestRequest()
        request.LANGUAGE = "en"
        form = TestAddForm(self.portal, request)
        form.update()

        self.widget = RecurrenceFieldWidget(form.fields["recurrence"].field, request)

        self.widget.form = form
        self.widget.start_field = "day"
        self.widget.update()

    def test_widget_options(self):
        pat_options = json.loads(self.widget.get_pattern_options())
        self.assertEqual(
            {
                "ajaxContentType": "application/x-www-form-urlencoded",
                "ajaxURL": "http://nohost/plone/@@json_recurrence",
                "firstDay": 7,
                "hasRepeatForeverButton": True,
                "lang": "en",
                "readOnly": False,
                "ributtonExtraClass": "allowMultiSubmit",
                "startField": "form.widgets.day",
            },
            pat_options["configuration"],
        )

    def test_widget_rendering(self):
        html = self.widget.render()
        self.assertIn("pat-recurrence", html)



from plone.formwidget.recurrence import _
from z3c.form.interfaces import IWidget
from zope.schema import ValidationError
from zope.schema.interfaces import IText


# Fields
class IRecurrenceField(IText):
    """Special marker for datetime fields that use our widget."""


# Widgets
class IRecurrenceWidget(IWidget):
    """Date widget marker for z3c.form."""


# Errors
class RecurrenceValidationError(ValidationError):
    __doc__ = _("The recurrence rule couldn't be parsed.")



from plone.formwidget.recurrence.z3cform.interfaces import IRecurrenceField
from zope import schema
from zope.interface import implementer


@implementer(IRecurrenceField)
class RecurrenceField(schema.Text):
    pass



from plone.formwidget.recurrence.browser.i18n import translations
from plone.formwidget.recurrence.z3cform.interfaces import IRecurrenceField
from plone.formwidget.recurrence.z3cform.interfaces import IRecurrenceWidget
from Products.CMFCore.utils import getToolByName
from z3c.form.browser.textarea import TextAreaWidget
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.widget import FieldWidget
from zope.component import adapter
from zope.component.hooks import getSite
from zope.interface import implementer
from zope.interface import implementer_only
from zope.traversing.browser import absoluteURL

import json


@implementer_only(IRecurrenceWidget)
class RecurrenceWidget(TextAreaWidget):
    """Recurrence widget implementation."""

    klass = "recurrence-widget"
    value = ""
    start_field = None
    show_repeat_forever = True

    def site_url(self):
        return absoluteURL(getSite(), self.request)

    @property
    def read_only(self):
        return self.mode == "display"

    def get_start_field(self):
        if self.mode == "display":
            return self.id + "-start"
        if hasattr(self.form.widgets[self.start_field], "js_field"):
            return self.form.widgets[self.start_field].js_field
        if hasattr(self.form.widgets[self.start_field], "name"):
            return self.form.widgets[self.start_field].name
        return self.form.widgets[self.start_field].id

    def get_start_date(self):
        start = self.form.fields[self.start_field].field.get(self.context)
        return start.strftime("%Y-%m-%d %H:%M")

    def first_day(self):
        """First day of the Week. 0..Sunday, 6..Saturday.

        .. Note::
            This value is likely to be overwritten by the widget configuration.
        """
        calendar = self.request.locale.dates.calendars["gregorian"]
        return calendar.week.get("firstDay", 0)

    def get_pattern_options(self):
        portal = getToolByName(getSite(), "portal_url").getPortalObject()
        ajax_url = portal.absolute_url() + "/@@json_recurrence"
        conf = dict(
            ajaxContentType="application/x-www-form-urlencoded",
            ajaxURL=ajax_url,
            firstDay=self.first_day(),
            hasRepeatForeverButton=self.show_repeat_forever,
            lang=self.request.LANGUAGE,
            readOnly=self.read_only,
            ributtonExtraClass="allowMultiSubmit",
            startField=self.get_start_field(),
        )
        return json.dumps(
            {
                "localization": translations(self.request),
                "language": self.request.LANGUAGE,
                "configuration": conf,
            }
        )


@implementer(IFieldWidget)
@adapter(IRecurrenceField, IFormLayer)
def RecurrenceFieldWidget(field, request):
    """IFieldWidget factory for RecurrenceWidget."""
    return FieldWidget(field, RecurrenceWidget(request))



from setuptools import setup


version = "4.0.0a1"

setup(
    name="plone.formwidget.recurrence",
    version=version,
    description="Recurrence widget for Plone",
    long_description="{}\n{}".format(
        open("README.rst").read(), open("CHANGES.rst").read()
    ),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: 6.2",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    keywords="Plone Event Recurrence Date Time Widget z3c.form",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://github.com/plone/plone.formwidget.recurrence",
    license="GPL",
    include_package_data=True,
    python_requires=">=3.10",
    zip_safe=False,
    install_requires=[
        "plone.base",
        "Products.GenericSetup",
        "python-dateutil",
        "Zope",
    ],
    extras_require=dict(
        z3cform=[
            "z3c.form",
        ],
        test=[
            "plone.app.testing",
            "plone.app.z3cform",
        ],
    ),
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



"""Base module for unittesting"""

from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer

import unittest


class PloneFormwidgetRecurrenceLayer(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        """Set up Zope."""

        import plone.app.z3cform

        self.loadZCML(package=plone.app.z3cform)

        import plone.formwidget.recurrence

        self.loadZCML(package=plone.formwidget.recurrence)

    def setUpPloneSite(self, portal):
        """Set up Plone."""
        self.applyProfile(portal, "plone.formwidget.recurrence:default")


FIXTURE = PloneFormwidgetRecurrenceLayer()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,), name="PloneFormwidgetRecurrenceLayer:Integration"
)
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name="PloneFormwidgetRecurrenceLayer:Functional"
)


class IntegrationTestCase(unittest.TestCase):
    """Base class for integration tests."""

    layer = INTEGRATION_TESTING


class FunctionalTestCase(unittest.TestCase):
    """Base class for functional tests."""

    layer = FUNCTIONAL_TESTING
